@@ -21,7 +21,7 @@
  *
  */
 
-//legacy engine
+/* legacy engine */
 #include <emerald.h>
 #include <engine.h>
 #include <legacy_icon.h>
@@ -57,10 +57,22 @@ typedef struct _private_ws
 
 void get_meta_info (EngineMetaInfo * emi)
 {
+    guint8 *pixbuf_data;
+
     emi->version = g_strdup("0.1");
-    emi->description = g_strdup(_("Based on original gnome-window-decorator"));
-    emi->last_compat = g_strdup("0.0"); // old themes still compatible
-    emi->icon = gdk_pixbuf_new_from_inline(-1, my_pixbuf, TRUE, NULL);
+    emi->description = g_strdup(_("Based on original gtk-window-decorator"));
+    /* old themes are marked still compatible */
+    emi->last_compat = g_strdup("0.0");
+
+    pixbuf_data = g_memdup(LEGACY_ICON_PIXEL_DATA,
+                           LEGACY_ICON_ROWSTRIDE * LEGACY_ICON_HEIGHT);
+    emi->icon = gdk_pixbuf_new_from_data(pixbuf_data, GDK_COLORSPACE_RGB,
+                                         (LEGACY_ICON_BYTES_PER_PIXEL != 3), 8,
+                                         LEGACY_ICON_WIDTH,
+                                         LEGACY_ICON_HEIGHT,
+                                         LEGACY_ICON_ROWSTRIDE,
+                                         (GdkPixbufDestroyNotify) g_free,
+                                         pixbuf_data);
 }
 
 void engine_draw_frame (decor_t * d, cairo_t * cr)
@@ -88,17 +100,17 @@ void engine_draw_frame (decor_t * d, cairo_t * cr)
 
     h = d->height - ws->top_space - ws->titlebar_height - ws->bottom_space;
 
-    int corners = 
+    int corners =
         ((pws->round_top_left)     ? CORNER_TOPLEFT     : 0) |
         ((pws->round_top_right)    ? CORNER_TOPRIGHT    : 0) |
         ((pws->round_bottom_left)  ? CORNER_BOTTOMLEFT  : 0) |
         ((pws->round_bottom_right) ? CORNER_BOTTOMRIGHT : 0);
-    
-	// maximize work-a-round
+
+	/* maximize work-a-round */
 	if (d->state & (WNCK_WINDOW_STATE_MAXIMIZED_HORIZONTALLY |
                 WNCK_WINDOW_STATE_MAXIMIZED_VERTICALLY))
         corners = 0;
-   
+
 	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
     cairo_set_line_width (cr, 1.0);
 
@@ -185,17 +197,17 @@ void engine_draw_frame (decor_t * d, cairo_t * cr)
             SHADE_BOTTOM | SHADE_RIGHT, ws,
             pws->corner_radius);
 
-	// ======= NEW LAYER =======
+    /* ======= NEW LAYER ======= */
     cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-   
-	// titlebar separator line
+
+    /* titlebar separator line */
     cairo_set_source_alpha_color(cr, &pfs->separator_line);
     cairo_move_to (cr, x1 + 0.5, y1 + top - 0.5);
     cairo_rel_line_to (cr, x2 - x1 - 1.0, 0.0);
     cairo_stroke (cr);
 
 
-	// do not draw outside the decoration area
+    /* do not draw outside the decoration area */
     rounded_rectangle (cr,
             x1 + 0.5, y1 + 0.5,
             x2 - x1 - 1.0, y2 - y1 - 1.0,
@@ -206,7 +218,7 @@ void engine_draw_frame (decor_t * d, cairo_t * cr)
 
     cairo_translate (cr, 1.0, 1.0);
 
-	// highlight
+    /* highlight */
     rounded_rectangle (cr,
             x1 + 0.5, y1 + 0.5,
             x2 - x1 - 1.0, y2 - y1 - 1.0,
@@ -220,7 +232,7 @@ void engine_draw_frame (decor_t * d, cairo_t * cr)
     cairo_translate (cr, -2.0, -2.0);
 
 
-	// shadow
+    /* shadow */
     rounded_rectangle (cr,
             x1 + 0.5, y1 + 0.5,
             x2 - x1 - 1.0, y2 - y1 - 1.0,
@@ -234,8 +246,8 @@ void engine_draw_frame (decor_t * d, cairo_t * cr)
     cairo_translate (cr, 1.0, 1.0);
 
     cairo_reset_clip (cr);
-	
-	// halo
+
+    /* halo */
     rounded_rectangle (cr,
             x1 + 0.5, y1 + 0.5,
             x2 - x1 - 1.0, y2 - y1 - 1.0,
@@ -246,8 +258,8 @@ void engine_draw_frame (decor_t * d, cairo_t * cr)
     cairo_set_source_alpha_color (cr, &pfs->window_halo);
     cairo_stroke (cr);
 
-    // inner border
-    //TODO - make this a bit more pixel-perfect...but it works for now
+    /* inner border
+       TODO: make this a bit more pixel-perfect... but it works for now */
 
     cairo_set_line_width (cr, 1.0);
 
@@ -276,7 +288,7 @@ void load_engine_settings(GKeyFile * f, window_settings * ws)
 {
     private_ws * pws = ws->engine_ws;
 
-	// parse color settings
+    /* parse color settings */
     PFACS(outer);
     PFACS(inner);
     PFACS(title_outer);
@@ -289,7 +301,7 @@ void load_engine_settings(GKeyFile * f, window_settings * ws)
     PFACS(contents_highlight);
     PFACS(contents_halo);
 
-	// parse border settings
+    /* parse border settings */
     load_bool_setting(f, &pws->round_top_left, "round_top_left", SECT);
     load_bool_setting(f, &pws->round_top_right, "round_top_right", SECT);
     load_bool_setting(f, &pws->round_bottom_left, "round_bottom_left", SECT);
@@ -303,20 +315,18 @@ void init_engine(window_settings * ws)
     private_fs * pfs;
     private_ws * pws;
 
-	// private window settings
-    pws = malloc(sizeof(private_ws));
+    /* private window settings */
+    pws = g_malloc0(sizeof(private_ws));
     ws->engine_ws = pws;
-    bzero(pws,sizeof(private_ws));
     pws->round_top_left = TRUE;
     pws->round_top_right = TRUE;
     pws->round_bottom_left = TRUE;
     pws->round_bottom_right = TRUE;
     pws->corner_radius = 5.0;
 
-	// private frame settings for active frames
-    pfs = malloc(sizeof(private_fs));
+    /* private frame settings for active frames */
+    pfs = g_malloc0(sizeof(private_fs));
     ws->fs_act->engine_fs = pfs;
-    bzero(pfs,sizeof(private_fs));
     ACOLOR(inner, 0.8, 0.8, 0.8, 0.5);
     ACOLOR(outer, 0.8, 0.8, 0.8, 0.5);
     ACOLOR(title_inner, 0.8, 0.8, 0.8, 0.8);
@@ -329,9 +339,8 @@ void init_engine(window_settings * ws)
     ACOLOR(contents_shadow, 0.6, 0.6, 0.6, 0.8);
     ACOLOR(contents_halo, 0.8, 0.8, 0.8, 0.8);
 
-	// private frame settings for inactive frames
-    pfs = malloc(sizeof(private_fs));
-    bzero(pfs, sizeof(private_fs));
+    /* private frame settings for inactive frames */
+    pfs = g_malloc0(sizeof(private_fs));
     ws->fs_inact->engine_fs = pfs;
     ACOLOR(inner, 0.8, 0.8, 0.8, 0.3);
     ACOLOR(outer, 0.8, 0.8, 0.8, 0.3);
@@ -348,8 +357,8 @@ void init_engine(window_settings * ws)
 
 void fini_engine(window_settings * ws)
 {
-    free(ws->fs_act->engine_fs);
-    free(ws->fs_inact->engine_fs);
+    g_free(ws->fs_act->engine_fs);
+    g_free(ws->fs_inact->engine_fs);
 }
 
 void layout_corners_frame(GtkWidget * vbox)
@@ -373,9 +382,13 @@ void layout_corners_frame(GtkWidget * vbox)
     gtk_box_pack_startC(vbox, junk, FALSE, FALSE, 0);
     register_setting(junk, ST_BOOL, SECT, "round_bottom_right");
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+#else
     hbox = gtk_hbox_new(FALSE, 2);
+#endif
     gtk_box_pack_startC(vbox, hbox, FALSE, FALSE, 0);
-    
+
     gtk_box_pack_startC(hbox, gtk_label_new(_("Rounding Radius")), FALSE, FALSE, 0);
 
     junk = scaler_new(0, 20, 0.5);
@@ -387,19 +400,31 @@ void my_engine_settings(GtkWidget * hbox, gboolean active)
 {
     GtkWidget * vbox;
     GtkWidget * scroller;
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
+#else
     vbox = gtk_vbox_new(FALSE, 2);
+#endif
     gtk_box_pack_startC(hbox, vbox, TRUE, TRUE, 0);
     gtk_box_pack_startC(vbox, gtk_label_new(active?"Active Window":"Inactive Window"), FALSE, FALSE, 0);
+#if GTK_CHECK_VERSION(3, 2, 0)
+    gtk_box_pack_startC(vbox, gtk_separator_new (GTK_ORIENTATION_HORIZONTAL), FALSE, FALSE, 0);
+#else
     gtk_box_pack_startC(vbox, gtk_hseparator_new(), FALSE, FALSE, 0);
+#endif
     scroller = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
             GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
     gtk_box_pack_startC(vbox, scroller, TRUE, TRUE, 0);
-    
+
     table_new(3, FALSE, FALSE);
 
+#if GTK_CHECK_VERSION(3, 8, 0)
+    gtk_container_add(GTK_CONTAINER(scroller), GTK_WIDGET(get_current_table()));
+#else
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroller), GTK_WIDGET(get_current_table()));
-    
+#endif
+
     make_labels(_("Colors"));
     table_append_separator();
     ACAV(_("Outer Frame Blend"), "outer", SECT);
@@ -422,10 +447,18 @@ void my_engine_settings(GtkWidget * hbox, gboolean active)
 void layout_engine_colors(GtkWidget * vbox)
 {
     GtkWidget * hbox;
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
+#else
     hbox = gtk_hbox_new(FALSE, 2);
+#endif
     gtk_box_pack_startC(vbox, hbox, TRUE, TRUE, 0);
     my_engine_settings(hbox, TRUE);
+#if GTK_CHECK_VERSION(3, 2, 0)
+    gtk_box_pack_startC(hbox, gtk_separator_new (GTK_ORIENTATION_VERTICAL), FALSE, FALSE, 0);
+#else
     gtk_box_pack_startC(hbox, gtk_vseparator_new(), FALSE, FALSE, 0);
+#endif
     my_engine_settings(hbox, FALSE);
 }
 
