@@ -9,9 +9,12 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 #include <X11/Xregion.h>
-#include <X11/cursorfont.h>
 #include <X11/extensions/Xrender.h>
+#ifdef HAVE_XINPUT2
+#include <X11/extensions/XInput2.h>
+#endif
 
+/*
 #ifndef GDK_DISABLE_DEPRECATED
 #define GDK_DISABLE_DEPRECATED
 #endif
@@ -19,6 +22,7 @@
 #ifndef GTK_DISABLE_DEPRECATED
 #define GTK_DISABLE_DEPRECATED
 #endif
+*/
 
 #include <gtk/gtk.h>
 #include <gtk/gtkwindow.h>
@@ -31,11 +35,8 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #endif
 
-//#include <gconf/gconf-client.h>
-
 #define WNCK_I_KNOW_THIS_IS_UNSTABLE
 #include <libwnck/libwnck.h>
-#include <libwnck/window-action-menu.h>
 
 #include <cairo.h>
 #include <cairo-xlib.h>
@@ -50,7 +51,7 @@
 #include <dlfcn.h>
 #include <glib.h>
 #include <glib/gstdio.h>
-#include <string.h>
+#include <glib/gprintf.h>
 #include <stdlib.h>
 #include <math.h>
 #include <limits.h>
@@ -58,8 +59,8 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <sys/wait.h>
-#include<libintl.h>
-#include<locale.h>
+#include <libintl.h>
+#include <locale.h>
 #define _(String) gettext (String)
 #define gettext_noop(String) String
 #define N_(String) gettext_noop (String)
@@ -108,7 +109,7 @@ typedef struct {
     unsigned long decorations;
 } MwmHints;
 
-//static double decoration_alpha = 0.5; //Decoration Alpha
+/* static double decoration_alpha = 0.5; // decoration alpha */
 
 #define SWITCHER_SPACE     40
 #define SWITCHER_TOP_EXTRA 4
@@ -155,9 +156,11 @@ typedef struct _window_settings
     gint button_offset;
     gint button_hoffset;
     gchar * tobj_layout;
-    
+
     gint double_click_action;
+    gint middle_click_action;
     gint button_hover_cursor;
+    gboolean titlebar_no_scroll_shade;
 
     gboolean round_top_left;
     gboolean round_top_right;
@@ -167,24 +170,23 @@ typedef struct _window_settings
     frame_settings * fs_act;
     frame_settings * fs_inact;
     gint min_titlebar_height;
-    gboolean use_pixmap_buttons;// = FALSE;
-    double	corner_radius;//	=	5.0;
-    PangoAlignment title_text_align;// = PANGO_ALIGN_CENTER;
-    GdkPixbuf * ButtonPix[S_COUNT*B_COUNT];
-    GdkPixbuf * ButtonArray[B_COUNT];
+    gboolean        use_pixmap_buttons;
+    PangoAlignment  title_text_align; /* = PANGO_ALIGN_CENTER; */
+    cairo_surface_t *button_surface[S_COUNT*B_COUNT];
+    cairo_surface_t *button_array[B_COUNT];
 
-    gboolean    use_button_glow;
-    gboolean    use_button_inactive_glow;
-	gboolean	use_decoration_cropping;
-    gboolean    use_button_fade;
-    GdkPixbuf * ButtonGlowPix[B_COUNT];
-    GdkPixbuf * ButtonGlowArray;
-    GdkPixbuf * ButtonInactiveGlowArray;
-    GdkPixbuf * ButtonInactiveGlowPix[B_COUNT];
-    int         button_fade_num_steps;        // number of steps
-    int         button_fade_step_duration;    // step duration in milliseconds
-    int         button_fade_pulse_len_steps;  // length of pulse (number of steps)
-    int         button_fade_pulse_wait_steps; // how much pulse waits before fade out
+    gboolean	    use_button_glow;
+    gboolean	    use_button_inactive_glow;
+    gboolean	    use_decoration_cropping;
+    gboolean	    use_button_fade;
+    cairo_surface_t *button_glow_surface[B_COUNT];
+    cairo_surface_t *button_glow_array;
+    cairo_surface_t *button_inactive_glow_surface[B_COUNT];
+    cairo_surface_t *button_inactive_glow_array;
+    int		    button_fade_num_steps;        /* number of steps */
+    int		    button_fade_step_duration;    /* step duration in milliseconds */
+    int		    button_fade_pulse_len_steps;  /* length of pulse (number of steps) */
+    int		    button_fade_pulse_wait_steps; /* how much pulse waits before fade out */
     /* = {
     { 0, 6, 16, 16,   1, 0, 0, 0 },
     { 0, 6, 16, 16,   1, 0, 0, 0 },
@@ -196,64 +198,60 @@ typedef struct _window_settings
     gint    shadow_color[3];
     gint    shadow_offset_x;
     gint    shadow_offset_y;
-    decor_extents_t shadow_extents;//   = { 0, 0, 0, 0 };
-    decor_extents_t win_extents;//      = { 6, 6, 4, 6 };
+    decor_extents_t shadow_extents; /* = { 0, 0, 0, 0 }; */
+    decor_extents_t win_extents; /* = { 6, 6, 4, 6 }; */
     pos_t pos[3][3];
-    gint left_space;//   = 6;
-    gint right_space;//  = 6;
-    gint top_space;//    = 4;
-    gint bottom_space;// = 6;
+    gint left_space; /* = 6; */
+    gint right_space; /* = 6; */
+    gint top_space; /* = 4; */
+    gint bottom_space; /* = 6; */
 
-    gint left_corner_space;//   = 0;
-    gint right_corner_space;//  = 0;
-    gint top_corner_space;//    = 0;
-    gint bottom_corner_space;// = 0;
+    gint left_corner_space;
+    gint right_corner_space;
+    gint top_corner_space;
+    gint bottom_corner_space;
 
-    gint titlebar_height;// = 17; //Titlebar Height
+    gint titlebar_height; /* = 17; // titlebar height */
 
-    gint normal_top_corner_space;//      = 0;
+    gint normal_top_corner_space;
 
-    gint shadow_left_space;//   = 0;
-    gint shadow_right_space;//  = 0;
-    gint shadow_top_space;//    = 0;
-    gint shadow_bottom_space;// = 0;
+    gint shadow_left_space;
+    gint shadow_right_space;
+    gint shadow_top_space;
+    gint shadow_bottom_space;
 
-    gint shadow_left_corner_space;//   = 0;
-    gint shadow_right_corner_space;//  = 0;
-    gint shadow_top_corner_space;//    = 0;
-    gint shadow_bottom_corner_space;// = 0;
+    gint shadow_left_corner_space;
+    gint shadow_right_corner_space;
+    gint shadow_top_corner_space;
+    gint shadow_bottom_corner_space;
 
 
-    GdkPixmap *shadow_pixmap;// = NULL;
-    GdkPixmap *large_shadow_pixmap;// = NULL;
-    GdkPixmap *decor_normal_pixmap;// = NULL;
-    GdkPixmap *decor_active_pixmap;// = NULL;
+    cairo_surface_t *shadow_surface;
+    cairo_surface_t *large_shadow_surface;
+    cairo_surface_t *decor_normal_surface;
+    cairo_surface_t *decor_active_surface;
 
-    cairo_pattern_t *shadow_pattern;// = NULL;
+    cairo_pattern_t *shadow_pattern;
 
-    gint		    text_height;
+    gint	    text_height;
 
     PangoFontDescription *font_desc;
     PangoContext * pango_context;
 
-    decor_extents_t switcher_extents;// = { 0, 0, 0, 0 };
-    GdkPixmap *switcher_pixmap;// = NULL;
-    GdkPixmap *switcher_buffer_pixmap;// = NULL;
-    gint      switcher_width;
-    gint      switcher_height;
-
-    gint switcher_top_corner_space;//    = 0;
-    gint switcher_bottom_corner_space;// = 0;
+    decor_extents_t switcher_extents; /* = { 0, 0, 0, 0 }; */
+    gint switcher_top_corner_space;
+    gint switcher_bottom_corner_space;
 
     struct _icon_size {
         int w, h;
     } c_icon_size[B_T_COUNT],
-      c_glow_size; // one glow size for all buttons
-                   // (buttons will be centered in their glows)
-                   // active and inactive glow pixmaps are assumed to be of same size
+      c_glow_size; /* one glow size for all buttons
+                      (buttons will be centered in their glows)
+                      active and inactive glow pixmaps are assumed to be of same
+                      size */
     gboolean stretch_sides;
-    gint blur_type;// = BLUR_TYPE_NONE;
-	
+    gint blur_type; /* = BLUR_TYPE_NONE; */
+
 } window_settings;
 
 struct _frame_settings
@@ -272,25 +270,26 @@ typedef struct _rectangle {
 
 typedef struct _button_fade_info
 {
-    gpointer * d; // needed by the timer function
+    gpointer * d; /* needed by the timer function */
     cairo_t * cr;
     double    y1;
-    int  counters[B_T_COUNT]; // 0: not fading, > 0: fading in, < 0: fading out
-                              // max value:  ws->button_fade_num_steps+1 (1 is reserved to indicate
-                              //                                          fade-in initiation)
-                              // min value: -ws->button_fade_num_steps
+    int  counters[B_T_COUNT]; /* 0: not fading, > 0: fading in, < 0: fading out
+                                 max value:  ws->button_fade_num_steps+1
+                                 (1 is reserved to indicate fade-in initiation)
+                                 min value: -ws->button_fade_num_steps */
     gboolean pulsating[B_T_COUNT];
     gint    timer;
     gboolean first_draw;
 } button_fade_info_t;
 
 typedef struct _button_region_t {
-    gint        base_x1, base_y1, base_x2, base_y2; // button coords with no glow
-    gint        glow_x1, glow_y1, glow_x2, glow_y2; // glow coordinates
+    gint        base_x1, base_y1, base_x2, base_y2; /* button coords with no glow */
+    gint        glow_x1, glow_y1, glow_x2, glow_y2; /* glow coordinates */
 
-    // holds whether this button's glow overlap with the other button's non-glow (base) area
+    /* holds whether this button's glow overlap with the other button's non-glow
+       (base) area */
     gboolean    overlap_buttons[B_T_COUNT];
-    GdkPixmap * bg_pixmap;
+    cairo_surface_t *bg_surface;
 } button_region_t;
 
 typedef struct _decor
@@ -300,12 +299,11 @@ typedef struct _decor
     guint	      button_states[B_T_COUNT];
     gint tobj_pos[3];
     gint tobj_size[3];
-    gint tobj_item_pos[11];
-    gint tobj_item_state[11];
-    gint tobj_item_width[11];
-    GdkPixmap	      *pixmap;
-    GdkPixmap	      *buffer_pixmap;
-    GdkGC	      *gc;
+    gint tobj_item_pos[TBT_COUNT];
+    gint tobj_item_state[TBT_COUNT];
+    gint tobj_item_width[TBT_COUNT];
+    cairo_surface_t   *surface;
+    cairo_surface_t   *buffer_surface;
     gint	      width;
     gint	      height;
     gint              client_width;
@@ -315,7 +313,7 @@ typedef struct _decor
     PangoLayout	      *layout;
     gchar	      *name;
     cairo_pattern_t   *icon;
-    GdkPixmap	      *icon_pixmap;
+    cairo_surface_t   *icon_surface;
     GdkPixbuf	      *icon_pixbuf;
     WnckWindowState   state;
     WnckWindowActions actions;
@@ -324,12 +322,14 @@ typedef struct _decor
     frame_settings * fs;
     void	      (*draw) (struct _decor *d);
     button_region_t   button_region[B_T_COUNT];
-    rectangle_t       min_drawn_buttons_region; // minimal rectangle enclosing all drawn regions
+    /* minimal rectangle enclosing all drawn regions */
+    rectangle_t       min_drawn_buttons_region;
     gboolean          draw_only_buttons_region;
-    gint              button_last_drawn_state[B_T_COUNT]; // last drawn state or fade counter
+    /* last drawn state or fade counter */
+    gint              button_last_drawn_state[B_T_COUNT];
     button_fade_info_t button_fade_info;
-    GdkPixmap * p_active, * p_active_buffer;
-    GdkPixmap * p_inactive, * p_inactive_buffer;
+    cairo_surface_t   *p_active_surface, *p_active_buffer_surface;
+    cairo_surface_t   *p_inactive_surface, *p_inactive_buffer_surface;
     button_region_t   button_region_inact[B_T_COUNT];
     gboolean only_change_active;
 } decor_t;
